require "test_helper"
require_relative "../../app/models/paginated"

class FakePaginated
  include Paginated
  DATA = ("a".."z").to_a

  # fake inspection
  attr_reader :data

  # fake a model
  def initialize
    @data = DATA
  end

  # fake a model
  def offset(start)
    @data = @data[start..]
    self
  end

  # fake a model
  def limit(count)
    @data = @data[...count]
    self
  end

  def count_by_page
    3
  end
end

class PaginatedTest < ActiveSupport::TestCase
  def assert_valid_pagination(paginated, count:)
    assert_equal count, paginated.data.size
  end

  test "paginated with placeholder" do
    test_empty_params = FakePaginated.new.paginate({})
    assert_valid_pagination test_empty_params, count: FakePaginated.new.count_by_page
    assert_equal %w[a b c], test_empty_params.data

    test_valid_params = FakePaginated.new.paginate({ page: 1 })
    assert_valid_pagination test_valid_params, count: FakePaginated.new.count_by_page
    assert_equal %w[d e f], test_valid_params.data
    test_valid_params = FakePaginated.new.paginate({ page: 4 })
    assert_valid_pagination test_valid_params, count: FakePaginated.new.count_by_page
    assert_equal %w[m n o], test_valid_params.data

    test_invalid_params = FakePaginated.new.paginate({ page: "what" })
    assert_valid_pagination test_invalid_params, count: FakePaginated.new.count_by_page
    assert_equal %w[a b c], test_invalid_params.data
  end
end
